 0v # SAE – Rapport Qualité de Developpement

### Rafael CAFÉ – Titouan DUPUY – Tristan DUCHEMIN – Jeanne VIAUD


Dans ce document seront présentés les différents design patterns utilisés pour mener à bien notre projet de SAE. Nous expliquerons pourquoi ceux-ci sont pertinents, et comment ils ont été intégrés à notre projet. Les design patterns utilisés ont été implémentés en PHP.


## DP1 - Factory

Ce design pattern nous et utile dans la conception de notre dp strategy. Celui-ci nous sert à choisir le mode de réduction que l'on veut, en entrant en paramètre une réduction (sur cette version c'est simplement un float mais pourquoi pas créer une énumération à l'avenir). 

Pour nous, aucun autre DP n'aurait pû remplir ce rôle. 

### Code pertinant
```php
public static function creerPrixMontre(float $prix, float $reduction):PrixMontre{
    if ($reduction = 50.0){
        return new ReductionCinquante($prix);
    }
    if ($reduction = 100.0){
        return new ReductionCent($prix);
    }
    if ($reduction = 0.0){
        return new PrixSimple($prix);
    }
    else{
        throw new \ErrorException('réduction inconnu');
    }
}
```

Pour l'UML, toutes les informations du DP Factory sont présentes sur l'UML du DP Strategy

## DP 2 - Strategy

Dans notre site de commerce, nous avons voulu mettre en place un système de réduction (tout le temps présent). Pour cela nous récupérons le prix de base d’un produit puis le soumettons à une réduction. Le DP Strategy est utilisé pour choisir cette dites réduction : La classe PrixMontre est abstraite, et donc la fonction calculePrix() n'est pas défini. Les classe filles vont donc créer elles-même la fonction calculPrix() appliquant la réduction. Ainsi, si le prix est un ReductionCinquante, calculPrix() renverra le prix_HR (hors réduction) * 0.5. 
Ce design pattern nous permet d'ajouter de nouveau type de réduction très facilement, en créant une classe héritante de PrixMontre qui redéfini seulement le calculPrix()

Au départ nous avions utilisé le DP Template Method, mais celui-ci propose vraiment une "complétion" d'algorithme, alors que nous cherchions à faire des méthodes différente de réduction. Par exemple avec un Template Method, nous aurions dû choisir un type de réduction particulier comme la réduction en pourcentage, et ne jamais changer.  
Strategy nous permet de modeler le prix tel que l'on souhaite.

### Code pertinant
```php
abstract class PrixMontre
{
public $prix_HR;
private function __construct(float $prix){
    $this->prix_HR = $prix;
}

public function changePrix(float $nouveauPrix){
    $this->prix_HR = $nouveauPrix;
}
abstract protected function calculPrix():float;


public static function creerPrixMontre(float $prix, float $reduction):PrixMontre{
    if ($reduction = 50.0){
        return new ReductionCinquante($prix);
    }
    if ($reduction = 100.0){
        return new ReductionCent($prix);
    }
    if ($reduction = 0.0){
        return new PrixSimple($prix);
    }
    else{
        throw new \ErrorException('réduction inconnu');
    }
}

}
```
Réduction de 50%
```php
class ReductionCinquante extends PrixMontre
{
    public function calculPrix():float{
        return $this->prix_HR * 0.0;
    } 
}
```
### UML
![strategy]( img/strategy.png "DP Strategy")



## DP 3 - States


Ce DP a été le plus utile dans la réalisation de notre site. En effet, il nous permet de changer facilement les états de notre header en fonction de la connexion du client sur le site (connecté, déconnecté, admin). La foncion render() renverra un morceau de code html en fonction de l'état, et qui sera utilisé dans une vue.

Pour changer l'état de notre header, un autre DP aurait pû être utilisé : le Chain of Responsability. Il nous aurait permis de simplement compléter un header existant au fur et à mesure des étapes de la chaine. Mais dans ce cas il aurait été plus difficile d'ajouter des "type de profil" avec des boutons changeants. Le DP State est plus adapté à nos yeux.

### Code pertinant
```php
class HeaderContext
{
    private HeaderState $state;

    public function __construct(HeaderState $state)
    {
        $this->state = $state;
    }

    public function setState(HeaderState $state): void
    {
        $this->state = $state;
    }

    public function render(): string
    {
        return $this->state->render();
    }
}


```
Un exemple de State
```php
class LoginState implements HeaderState
{
    public function render(): string
    {
        return '
            <a href="' . site_url('logout') . '">Déconnexion</a>
        ';
    }
}
```
Comparaison avec le state admin
```php
class LoginAdminState implements HeaderState
{
    public function render(): string
    {
        return '
        <form method="post" action="' . site_url('admin_page') . '">
            <select name="choice" id="choice" onchange="this.form.submit()">
                <option value="">--Tables--</option>
                <option value="Montre">Montre</option>
                <option value="Matiere">Matiere</option>
                <option value="Marque">Marque</option>
                <option value="Adresse">Adresse</option>
                <option value="Client">Client</option>
                <option value="InfoBanquaires">InfoBanquaires</option>
                <option value="Commande">Commande</option>
            </select>
        </form>
        <a href="' . site_url('logout') . '">Déconnexion</a>
    ';
    }
}
```
### UML
![state]( img/state.png "DP state")
