<?php

namespace App\Controllers;

use App\Models\AdresseModele;
use App\Models\ClientModele;
use App\Models\CommandeModele;
use App\Models\InfoBanquairesModel;
use App\Models\MarqueModel;
use App\Models\MatiereModel;
use App\Models\MontreModele;
use App\Models\StyleModele;
use CodeIgniter\Database\Exceptions\DatabaseException;
use PhpParser\Node\Stmt\Return_;


class AdminController extends BaseController
{
    public function __construct()
    {
        $this->Home = new Home();

    }
    // Insert dans la base de donnée les information récupérées dans la Vue (post)
    // Certaines informations peuvent rester null (les informations obligatoires sont indiquées "NOTNULL" dans la BD)
    public function Insertion()
    {
        $data = $this->request->getPost();
        

        assert(isset($data)); // DEV EFFICACE

        if (isset($data)) {
            try {
                $Table = $data['Table'];
                $modele = $this->getModele($Table);
                $Vue = $this->getVue($Table);
                $row = array_slice($data,1);
                $modele->insert($row);
                $data = $modele->findAll();
                $data = array('data' => $data);
                return $this->Home->getHeader() . view($Vue, $data);
            } catch (DatabaseException $e) {
                return $this->Home->getHeader() . view('Erreur');
            }

        }


    }
    // Récupère la vue et le modèle en fonction du choix de l'admin. Le choix est un nom de table.
    public function admin_page()
    {
        $nom = $this->request->getGet('choice');
        $table =$this->getModele($nom);
        $data = $table->findAll();
        $vue = $this->getVue($nom);
        $data = array('data' => $data);
        return $this->Home->getHeader() . view($vue, $data);
    }

    // créer le modèle correspondant au nom de table donné en paramètre
    public function getModele($Table): AdresseModele|ClientModele|CommandeModele|InfoBanquairesModel|MarqueModel|MatiereModel|MontreModele|StyleModele
    {
        switch ($Table) {
            case 'Montre':
                return new MontreModele();
            case 'Matiere':
                return new MatiereModel();
            case 'Marque':
                return new MarqueModel();
            case 'Adresse':
                return new AdresseModele();
            case 'Client':
                return new ClientModele();
            case 'InfoBanquaires':
                return new InfoBanquairesModel();
            case 'Commande':
                return new CommandeModele();
            case 'Style':
                return new StyleModele();
        }
    }
    // donne un nom de vue correspondant au nom de table donné en paramètre
    public function getVue($Table)
    {
        switch ($Table) {
            case 'Montre':
                return 'AffichageProduits';
            case 'Matiere':
                return 'AffichageMatiere';
            case 'Marque':
                return 'AffichageMarque';
            case 'Adresse':
                return 'AffichageAdresse';
            case 'Client':
                return 'AffichageClient';
            case 'InfoBanquaires':
                return 'AffichageInfoBanquaires';
            case 'Commande':
                return 'AffichageCommande';
            case 'Style':
                return 'AffichageStyle';

        }
    }

    // supprime un produit de la base de donnée qui correspond à l'information donné par l'admin dans la vue
    public function Deletion()
    {
        $data = $this->request->getPost();

        assert(isset($data)); // DEV EFFICACE


        if (isset($data)) {
            try {
                $Table = $data['Table'];
                $modele = $this->getModele($Table);
                $Vue = $this->getVue($Table);
                $row = array_slice($data, 0);
                $modele->delete($row);
                $data = $modele->findAll();
                $data = array('data' => $data);

                return $this->Home->getHeader() . view($Vue, $data);
            } catch (DatabaseException $e) {
                return view('Erreur', array('message' => $e->getMessage()));
            }
        }
    }
    //met à jour un produit de la base de donnée qui correspond à l'information donné par l'admin dans la vue
    public function Update()
    {
        $data = $this->request->getPost();

        assert(isset($data)); // DEV EFFICACE

        if (isset($data)) {
            try {
                $Table = $data['Table'];
                $id = $data['ID_Montre'];
                $modele = $this->getModele($Table);
                $Vue = $this->getVue($Table);
                $row = array_slice($data, 2 );
                var_dump($row);
                $modele->update($id,$row);
                $data = $modele->findAll();
                $data = array('data' => $data);

                return $this->Home->getHeader() . view($Vue, $data);
            } catch (DatabaseException $e) {
    
                return $this->Home->getHeader() . view('Erreur');
            }

        }

    }

}
