<?php

namespace App\Controllers;
use App\Models\ClientModele;
use App\Models\MarqueModel;
use App\Models\MatiereModel;
use App\Models\MontreModele;
use App\Models\CommandeModele;
use App\Libraries\State\HeaderContext;
use App\Libraries\State\LoginState;
use App\Libraries\State\LoginAdminState;
use App\Libraries\State\LogoutState;
use App\Models\StyleModele;

class Home extends BaseController
{   
    //utilise un Template method State, choisi le header correspondant puis l'envoie à la vue. 
    public function getHeader(): string
    {
        $commandeModel = new CommandeModele();
        $clientModele = new ClientModele();
        // Vérifier l'état de connexion
        $session = session();
        $userRole = $session->get('newsession')['role']?? " "; // Par exemple : 'admin', 'user', ou null pour déconnecté
        // Définir l'état du header
        if ($userRole == 'admin') {
            $state = new LoginAdminState();
        } elseif ($userRole == 'user') {
            $state = new LoginState();
        } else {
            $state = new LogoutState();
        }
        $headerContext = new HeaderContext($state);

        //récupère la taille du panier pour l'afficher dans la vue header
        $taillePanier = 0;
        $email = $session->get('newsession')['identifiant'] ?? " ";
        if ($email != " "){
            $id = $clientModele->getClientIdByEmail($email);
            $id = $id->ID_Client;
            $commandes = $commandeModel->getCommande($id,null, 'En Cours');
            $taillePanier = count($commandes);
        }

        // Générer et retourner le contenu du header
        return view('header',['header'=>$headerContext->render(),'taillePanier'=>$taillePanier]);
    }
    //récupère toute les données de la base de donnée pour les envoyer à la vue home. Les données envoyés sont différentes si une catégorie est choisi
    public function index(): string
    {
        $montreModele = new MontreModele();
        $matiereModele = new MatiereModel();
        $marqueModele = new MarqueModel();
        $styleModele = new StyleModele();

        $data = $montreModele->findAll();
        $datamatiere = $matiereModele->findAll();
        $datamarque = $marqueModele->findAll();
        $datastyle = $styleModele->findAll();

        $categorie = $this->request->getGet();

        if (!empty($categorie)) {
            $data = $this->applyFilters($data, $categorie);
        }
        foreach ($categorie as $key => $value) {
            if ($value === "") {
                $categorie[$key] = null;
            }
        }

        return $this->getHeader() .view('home', [
            'data' => $data,
            'matieres' => $datamatiere,
            'marques' => $datamarque,
            'styles' => $datastyle,
            'filter' => $categorie,
        ]);
    }


    //modifie les données en entré en fonction d'un filtre donné
    private function applyFilters(array $data, array $filters): array
    {
        // Filtrage par style, marque, matière
        if (!empty($filters['stylechoice'])) {
            $data = array_filter($data, fn($montre) => $montre->ID_Style == $filters['stylechoice']);
        }

        if (!empty($filters['marquechoice'])) {
            $data = array_filter($data, fn($montre) => $montre->ID_Marque == $filters['marquechoice']);
        }

        if (!empty($filters['matierechoice'])) {
            $data = array_filter($data, fn($montre) => $montre->ID_Matiere == $filters['matierechoice']);
        }

        // Tri par prix
        if (!empty($filters['prixchoice'])) {
            $direction = $filters['prixchoice'] === "Croissant" ? 1 : -1;
            usort($data, fn($a, $b) => $direction * ($a->Prix <=> $b->Prix));
        }

        return $data;
    }

    //envoie sur la page correspondante à l'action renseignée par la vue.
    public function Connexion(): string
    {
        $action = $this->request->getPost('action');
        if ($action === 'Connexion') {
            return $this->login_page();
        } elseif ($action === 'Inscription') {
            return $this->register_page();
        }

        // Si aucune action reconnue
        return "Action non reconnue";
    }
    //envoie sur la page de login (avec persistance de l'identifiant)
    public function login_page(): string
    {
        $session = session();
        $identifiant = $session->get('newsession')['identifiant'] ?? '';

        return $this->getHeader() . view('login', ['identifiant' => $identifiant]);
    }
    //envoie sur la page d'inscription
    public function register_page(): string
    {
        return $this->getHeader() . view('register');
    }
    //envoie sur la page "à propos de nous"
    public function AboutUs() : string
    {
        return $this->getHeader().view('AboutUs');
    }
    
    public function Categorie(){
        $data = $this->request->getPost();
        $table = $data['Table'];
    
        $AdminController = new AdminController();
        $model = $AdminController->getModele($table);
    }


    

    


}


