<?php

namespace App\Controllers;

use App\Models\ClientModele;
use App\Models\CommandeModele;
use App\Models\MarqueModel;
use App\Models\MatiereModel;
use App\Models\MontreModele;
use App\Models\StyleModele;

class MontreControlleur extends BaseController
{
    private $modeleMontre;
    private $modeleMarque;
    private $modeleMatiere;
    private $clientModele;

    public function __construct()
    {
        $this->Home = new Home();
        $this->Paiemeant = new PaiementControlleur();
        $this->CommandeModele = new CommandeModele();

        // Initialisation des modèles
        $this->modeleMontre = new MontreModele();
        $this->modeleMarque = new MarqueModel();
        $this->modeleMatiere = new MatiereModel();
        $this->clientModele = new ClientModele();
        $this->modeleStyle = new StyleModele();
    }
    //récupère et envoie à la vu les infos correspondantes à la montre
    public function Montre()
    {
        $post = $this->request->getGet();

        if (!isset($post['ID_Montre'])) {
            return redirect()->to('/error_page'); // Rediriger en cas d'ID manquant
        }

        $session = session();
        $product = $post['ID_Montre'];

        // Stocker l'ID du produit dans la session
        $currentSession = $session->get('newsession', []);
        $currentSession['pageMontre'] = $product;
        $session->set('newsession', $currentSession);

        $montre = $this->modeleMontre->find($product);
        $id_style = $montre->ID_Style;
        $style = $this->modeleStyle->where('ID_Style', $id_style)->first();


        $marque = $this->modeleMarque->find($montre->ID_Marque);
        $matiere = $this->modeleMatiere->find($montre->ID_Matiere);

        $montre->ID_Style = $style['Nom'];
        $montre->ID_Marque = $marque->Nom;
        $montre->ID_Matiere = $matiere->Materiaux;


        return $this->Home->getHeader() . view('Article', ['montre' => $montre]);
    }
    // gère les redirection pour les actions disponibles pour la montre, ainsi que les data à envoyer.
    public function MontreAction()
    {
        $post = $this->request->getGet();

        if (!isset($post['ID_Montre'], $post['action'])) {
            return redirect()->to('/error_page'); // Rediriger en cas de données manquantes
        }

        $session = session();
        $email = $session->get('newsession')['identifiant'] ?? '';

        if ($email === '') {
            return redirect()->to('/login_page'); // Rediriger si l'utilisateur n'est pas connecté
        }

        $id_montre = $post['ID_Montre'];
        $quantity = $post['quantity'] ?? null;
        $date = date('Y-m-d');

        $client = $this->clientModele->getClientIdByEmail($email);
        $commandeExistante = $this->CommandeModele->getCommande($client->ID_Client, $id_montre, 'En Cours');

        $montre = $this->modeleMontre->find($id_montre);
        if (!$montre) {
            return redirect()->to('/error_page'); // Rediriger si la montre n'existe pas
        }

        $montreQuantity = $montre->Quantite;

        if ($commandeExistante) {
            $nouvelleQuantite = $commandeExistante[0]->Quantite + $quantity;
            if ($nouvelleQuantite > $montreQuantity) {
                $nouvelleQuantite = $montreQuantity;
            }
            $this->CommandeModele->update($commandeExistante[0]->ID_Commande, [
                'Quantite' => $nouvelleQuantite,
                'Date_Commande' => $date,
            ]);
        } else {
            $commande = [
                'Table' => 'Commande',
                'ID_Client' => $client->ID_Client,
                'ID_Montre' => $id_montre,
                'Quantite' => $quantity,
                'Etat' => 'En Cours',
                'Date_Commande' => $date,
            ];
            $this->CommandeModele->insert($commande);
        }

        if ($post['action'] === 'buy') {
            return $this->Paiemeant->paiement($id_montre);
        }

        return redirect()->to('/Montre?ID_Montre=' . $id_montre);

    }
}