<?php

namespace App\Controllers;

use App\Models\AdresseModele;
use App\Models\ClientModele;
use App\Models\CommandeModele;
use App\Models\InfoBanquairesModel;
use App\Models\MontreModele;

class PaiementControlleur extends BaseController
{
    public function __construct()
    {
        $this->Home = new Home();
        $this->clientModele = new ClientModele();
        $this->montreModel = new MontreModele();
        $this->CommandeModele = new CommandeModele();
        $this->AdresseModele = new AdresseModele();
        $this->InfoBanquairesModel = new InfoBanquairesModel(); 
    }


    public function paiement($id_montre = null)
    {
        $id_montre = $id_montre ?? $this->request->getGet('ID_Montre');

        $session = session();
        $email = $session->get('newsession')['identifiant'] ?? " ";

        $client = $this->clientModele->getClientIdByEmail($email);
        $id_client = $client->ID_Client;

        $client = $this->clientModele->find($id_client);

        $commandes = $this->CommandeModele->getCommande($id_client, $id_montre, 'En Cours');

        $montres = [];
        foreach ($commandes as $commande) {
            $montre = $this->montreModel->find($commande->ID_Montre);
            if ($montre) {
                $montres[] = $montre;
            }
        }

        $adresse = $this->AdresseModele->find($client->ID_Adresse);

        $infoBanquaire = $this->InfoBanquairesModel->find($client->Numero_Carte);
        $data = [
            'montres' => $montres,
            'commandes' => $commandes,
            'client' => $client,
            'adresse' => $adresse,
            'infoBanquaire' => $infoBanquaire,
        ];

        return $this->Home->getHeader() . view('paiement', $data);
    }



    //Modifie la base de donnée en fonction des acrticles acheter
    public function ProcessPayment()
    {
        $data = $this->request->getPost();

        // Vérifier que toutes les données nécessaires sont présentes
        $requiredFields = [
            'Nom' => 'Nom',
            'Prenom' => 'Prénom',
            'Numero_telephone' => 'Numéro de téléphone',
            'Rue' => 'Rue',
            'Ville' => 'Ville',
            'Code_Postal' => 'Code postal',
            'Pays' => 'Pays',
            'Numero_Carte' => 'Numéro de carte bancaire',
            'Date_Expiration' => 'Date d\'expiration',
        ];

        // Vérification des champs manquants
        $missingFields = [];
        foreach ($requiredFields as $field => $fieldName) {
            if (empty($data[$field])) {
                $missingFields[] = $fieldName;
            }
        }

        // Si des champs sont manquants, afficher un message d'erreur
        if (!empty($missingFields)) {
            $errorMessage = 'Les champs suivants sont manquants : ' . implode(', ', $missingFields);
            die($errorMessage);
        }

        // Extraction des données du formulaire
        $Nom = $data['Nom'];
        $Prenom = $data['Prenom'];
        $Num = $data['Numero_telephone'];
        $Rue = $data['Rue'];
        $Ville = $data['Ville'];
        $Code = $data['Code_Postal'];
        $Pays = $data['Pays'];
        $NumC = $data['Numero_Carte'];
        $date = $data['Date_Expiration'];

        // Récupérer l'utilisateur connecté
        $session = session();
        $email = $session->get('newsession')['identifiant'] ?? " ";
        $client = $this->clientModele->getClientIdByEmail($email);
        $id_client = $client->ID_Client;
        $client = $this->clientModele->find($id_client);

        // Adresse - Vérifier si elle existe déjà dans la base de données
        $adresseExistante = $this->AdresseModele->where([
            'Rue' => $Rue,
            'Ville' => $Ville,
            'Code_Postal' => $Code,
            'Pays' => $Pays,
        ])->first();

        if (!$adresseExistante) {
            // Si l'adresse n'existe pas, l'insérer
            $this->AdresseModele->insert([
                'Rue' => $Rue,
                'Ville' => $Ville,
                'Code_Postal' => $Code,
                'Pays' => $Pays,
            ]);
            $adresseID = $this->AdresseModele->getInsertID(); // Récupérer l'ID de l'adresse insérée
        } else {
            $adresseID = $adresseExistante->ID_Adresse;
        }

        // Créer une commande pour chaque article en cours
        $commandes = $this->CommandeModele->getCommande($id_client, null, 'En Cours');
        foreach ($commandes as $commande) {

            $montre = $this->montreModel->find($commande->ID_Montre);
            $nouvelleQuantite = $montre->Quantite - $commande->Quantite;
            // Mettre à jour la quantité disponible dans la base de données
            $this->montreModel->update($commande->ID_Montre, ['Quantite' => $nouvelleQuantite]);




            $commande->ID_Adresse_Livraison = $adresseID; // Assigner l'ID de l'adresse
            $commande->etat = 'Payée'; // Marquer comme payée
            $commandeArray= array(
                'ID_Client'=>$commande->ID_Client,
                'ID_Montre'=>$commande->ID_Montre,
                'Quantite'=>$commande->Quantite,
                'Etat'=>$commande->etat,
                'Date_Commande'=>$commande->Date_Commande,
                'ID_Adresse_Livraison'=> $commande->ID_Adresse_Livraison);

            // Mise à jour de la commande
            $this->CommandeModele->update($commande->ID_Commande,$commandeArray );
        }

        // Informations bancaires - Vérifier si elles existent déjà avant d'insérer
        $infoBanquaireExistante = $this->InfoBanquairesModel->find($client->Numero_Carte);
        if ($infoBanquaireExistante == null) {
            $this->InfoBanquairesModel->insert([
                'Numero_Carte' => $NumC,
                'Date_Expiration' => $date,
                'Nom' => $Nom,
            ]);
        }

        // Mettre à jour les informations du client
        $clientUpdate = [
            'Nom' => $Nom,
            'Prenom' => $Prenom,
            'Numero_telephone' => $Num,
            'Numero_Carte' => $NumC,
            'ID_Adresse' => $adresseID, // L'adresse récupérée ou insérée
        ];
        $this->clientModele->update($id_client, $clientUpdate);

        // Retourner une vue de confirmation
        return view('paiementAccepted', [
            'client' => $client
       ]);
    }
}

?>