<?php

namespace App\Controllers;

use App\Models\ClientModele;
use App\Models\CommandeModele;
use App\Models\MarqueModel;
use App\Models\MatiereModel;
use App\Models\MontreModele;

class PanierControlleur extends BaseController {

    protected $clientModele;
    protected $montreModel;
    protected $CommandeModele;
    protected $Home;

    public function __construct()
    {
        // Initialisation des modèles
        $this->clientModele = new ClientModele();
        $this->montreModel = new MontreModele();
        $this->CommandeModele = new CommandeModele();
        $this->Home = new Home();
    }

    //récupère les commandes grace à l'identifiant de la session et les affiche. 
    public function Cart()
    {
        $session = session();
        $email = $session->get('newsession')['identifiant'] ?? " ";

        if ($email == " ") {
            return redirect()->to('/login_page');
        }

        // Récupérer l'ID du client
        $client = $this->clientModele->getClientIdByEmail($email);


        // Récupérer les commandes en cours

        $carts = $this->CommandeModele->where('ID_Client', $client->ID_Client)
            ->where('Etat', 'En Cours')
            ->findAll();

        // Récupérer les montres associées aux commandes
        $montres = [];
        foreach ($carts as $cart) {
            $montre = $this->montreModel->find($cart->ID_Montre);
            if ($montre) {
                $montres[] = $montre;
            }
        }

        // Passer les données à la vue
        $array = ['carts' => $carts, 'montres' => $montres];
        return $this->Home->getHeader() . view('Cart', $array);
    }
    // supprime une commande
    public function removeItem()
    {
        $data = $this->request->getPost('ID_Montre');

        if (isset($data) && is_numeric($data)) {
            $deleted = $this->CommandeModele->where('ID_Montre', $data)->delete();

        }
        return redirect()->to('/Cart');
    }

    // modifie la quantité d'une commande. 
    public function updateQuantity()
    {
        $post = $this->request->getPost();
        assert(isset($post));

        $session = session();
        $email = $session->get('newsession')['identifiant'] ?? '';
        $quantity = $post['Quantite'] ?? null;
        $id_montre = $post['ID_Montre'];
        $date = date('Y-m-d');
        $client = $this->clientModele->getClientIdByEmail($email);

        // Vérifier si une commande pour ce client et cette montre existe déjà
        $commandeExistante = $this->CommandeModele->where('ID_Client', $client->ID_Client)
            ->where('ID_Montre', $id_montre)
            ->where('Etat', 'En Cours') // Optionnel, si tu veux seulement les commandes en cours
            ->first();

        $this->CommandeModele->update($commandeExistante->ID_Commande, [
            'Quantite' => $quantity,
            'Date_Commande' => $date, // On met à jour la date
        ]);
        return redirect()->to('/Cart');

    }

}
